//===============================================================================
// Microsoft Architecture Strategy Team
// LitwareHR - SaaS Sample Application
//===============================================================================
// Copyright  Microsoft Corporation.  All rights reserved.
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY
// OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT
// LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
// FITNESS FOR A PARTICULAR PURPOSE.
//===============================================================================
// The example companies, organizations, products, domain names,
// e-mail addresses, logos, people, places, and events depicted
// herein are fictitious.  No association with any real company,
// organization, product, domain name, email address, logo, person,
// places, or events is intended or should be inferred.
//===============================================================================

using System;
using System.Data;
using System.Configuration;
using System.Collections;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;
using LitwareHR.PublicApp.Presenters.Careers.Evaluation;
using LitwareHR.Recruiting.Contract;
using System.Threading;

public partial class Careers_Evaluation_Interview : BasePage, IInterviewView
{
    InterviewPresenter _presenter;
    InterviewPresenter Presenter
    {
        get
        {
            return _presenter;
        }
    }

    protected override void OnInitComplete(EventArgs e)
    {
        base.OnInitComplete(e);
        _presenter = new InterviewPresenter(this);
    }

    protected void Page_Load(object sender, EventArgs e)
    {
        _presenter.OnViewLoaded();
    }

    public void SetValidators()
    {
        if (!Page.IsPostBack)
        {
            DateRange.MaximumValue = DateTime.Now.ToShortDateString();
            DateRange.MinimumValue = DateTime.MinValue.ToShortDateString();
        }
    }

    public void SetDefaultValues()
    {
        if (!Page.IsPostBack)
        {
            string datePattern = Thread.CurrentThread.CurrentCulture.DateTimeFormat.ShortDatePattern;
            DateTextBox.Text = DateTime.Now.ToShortDateString();
            dateFormatLabel.Text = datePattern;
        }
    }

    protected void SubmitButton_Click(object sender, EventArgs e)
    {
        string workflowId = Request.QueryString[Constants.WorkflowIdQS];

        if (string.IsNullOrEmpty(workflowId))
            throw new ArgumentNullException(Constants.WorkflowIdQS);

        Page.Validate();

        if (Page.IsValid)
        {

            Interview interview = new Interview();

            DateTime date;
            DateTime.TryParse(DateTextBox.Text, out date);
            interview.Date = date;

            int duration;
            Int32.TryParse(DurationTextBox.Text, out duration);
            interview.Duration = duration;

            interview.Goals = GoalsTextBox.Text;
            interview.Notes = NotesTextBox.Text;
            interview.Positive = PositiveCheckBox.Checked;

            interview.WorkflowId = new Guid(workflowId);

            _presenter.OnSubmit(interview);
            string redirect = ConfigurationManager.AppSettings[Constants.RecruitingHome];
            Response.Redirect(redirect);
        }
    }
}
