//===============================================================================
// Microsoft Architecture Strategy Team
// LitwareHR - SaaS Sample Application
//===============================================================================
// Copyright  Microsoft Corporation.  All rights reserved.
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY
// OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT
// LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
// FITNESS FOR A PARTICULAR PURPOSE.
//===============================================================================
// The example companies, organizations, products, domain names,
// e-mail addresses, logos, people, places, and events depicted
// herein are fictitious.  No association with any real company,
// organization, product, domain name, email address, logo, person,
// places, or events is intended or should be inferred.
//===============================================================================

using System;
using System.Collections.Generic;
using System.Text;
using System.Runtime.Serialization;

namespace Shp.Security.BrokeredReceiver
{
    /// <summary>
    /// The contract that defines the application faults handled in the
    /// <see cref="ISecurityTokenService"/> contract.
    /// </summary>
    [DataContract(Namespace = Constants.SecurityTokenService.FaultNamespace, Name = Constants.SecurityTokenService.TrustFaultName)]
    public class TrustFault
    {
        private string message;
        private Guid id;

        /// <summary>
        /// Initializes a new instance of the <see cref="T:TrustFault"/> class.
        /// </summary>
        /// <param name="message">The message.</param>
        public TrustFault(string message)
            : this(message, Guid.NewGuid())
        {
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="T:TrustFault"/> class.
        /// </summary>
        /// <param name="message">The message.</param>
        /// <param name="id">The id.</param>
        public TrustFault(string message, Guid id)
        {
            this.message = message;
            this.id = id;
        }

        /// <summary>
        /// Gets or sets the fault message.
        /// </summary>
        /// <value>The fault message.</value>
        [DataMember(IsRequired = true, Name = "Message", Order = 0)]
        public string Message
        {
            get { return message; }
            set { message = value; }
        }

        /// <summary>
        /// Gets or sets the fault Id.
        /// </summary>
        /// <value>The fault id.</value>
        [DataMember(IsRequired = false, Name = "Id", Order = 1)]
        public Guid Id
        {
            get { return id; }
            set { id = value; }
        }
    }
}
